@@ -546,7 +546,7 @@ def _get_random_size_fractions(
     return height_fractions, width_fractions
 
 
-def _get_random_crop_inputs(
+def _get_crop_inputs(
         image_height: int,
         image_width: int,
         height_fraction_range: Tuple[float, float],
@@ -558,8 +558,6 @@ def _get_random_crop_inputs(
     """
     height_fraction_range, width_fraction_range: in range [0.0, 1.0)
     rand_fn: generate array of random numbers in range [0.0, 1.0)
-    Return: randomized (offset_heights, offset_widths,
-                        cropped_image_heights, cropped_image_widths)
     """
     image_height = convert_fn(image_height)
     image_width = convert_fn(image_width)
@@ -571,7 +569,7 @@ def _get_random_crop_inputs(
     )
 
     cropped_image_heights = image_height * height_fractions
-    cropped_image_widths = image_height * width_fractions
+    cropped_image_widths = image_width * width_fractions
 
     offset_heights = round_to_int_fn(
         (image_height - cropped_image_heights) * rand_fn()
